from itertools import ifilter

from edge.com import fs
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgconf.unity.bundle import (anpr, bus, essentials, speedcam, waga, mmr,
                                                        recipient, leddar, wiegand, sbr)
from edge.webserver.sysconf.dgconf.unity.builder import UnityConfBuilder
from edge.webserver.sysconf.dgmod.creator import BaseCreator


# noinspection PyUnusedLocal
def lr_get_creators(entry_uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type entry_uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list of BaseCreator
    """
    if entry_uimod.type != ph.UI_TYPE_LPR:
        return []

    try:
        cam_uimod = next(ifilter(lambda x: x.type in [ph.UI_TYPE_MV_CAMERA, ph.UI_TYPE_RTSP_CAMERA], all_used_uimods))
        lpr_uimod = next(ifilter(lambda x: x.type == ph.UI_TYPE_LPR, all_used_uimods))
        fusion_uimod = next(ifilter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods))
        tcp_recipient_uimods = filter(lambda x: x.type == ph.UI_TYPE_TCP_RECIPIENT, all_used_uimods)
        ledar_uimods = filter(lambda x: x.type == ph.UI_TYPE_LEDDAR, all_used_uimods)
        location_uimods = filter(lambda x: x.type == ph.UI_TYPE_LOCATION, all_used_uimods)
        ipcom_waga_uimods = filter(lambda x: x.type == ph.UI_TYPE_IPCOM_WAGA, all_used_uimods)

        activate = any([
            uimod_to_newconf_map.get(lpr_uimod),
            uimod_to_newconf_map.get(cam_uimod),

            any(map(lambda x: uimod_to_newconf_map.get(x), tcp_recipient_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), ledar_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), location_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), ipcom_waga_uimods))
        ])

        if forced or activate:
            return [LrCreator(lpr_uimod, cam_uimod, fusion_uimod, tcp_recipient_uimods, ledar_uimods,
                              location_uimods, ipcom_waga_uimods)]

    except StopIteration:
        pass

    return []


UNIT__ESSENTIALS0__API0 = 'essentials.0~api.0'
UNIT__ESSENTIALS0__COUNTERS0 = 'essentials.0~counters.0'
UNIT__MEDIA_SERVER0__MEDIA0 = 'media-server.0~media.0'
UNIT__EVENT_EMITTER0__TX_TRACK_BUS0 = 'event-emitter.0~tx-track-bus.0'
UNIT__EVENT_EMITTER0__TX_SCENE_BUS0 = 'event-emitter.0~tx-scene-bus.0'
UNIT__ANPR0__ANPR0 = 'anpr.0~anpr.0'
UNIT__MMR0__MMR0 = 'mmr.0~mmr.0'
UNIT__SBR0__SBR0 = 'sbr.0~sbr.0'
UNIT__SPEEDCAM0__SPEEDCAM0 = 'speedcam.0~bracket.0'
UNIT__IPCOM0_WAGA0 = 'ipcom.0~waga.0'
UNIT__LEDDAR0__LEDDAR0 = 'leddar.0~leddar.0'
UNIT__WIEGAND_PCB0__RECIPIENT0 = 'wiegand-pcb.0~recipient.0'
UNIT__WIEGAND_UDP0__RECIPIENT0 = 'wiegand-udp.0~recipient.0'
UNIT__RULE_PCB0__RULE_PCB0 = 'rule.pcb0~rule.pcb0'
UNIT__RULE_UDP0__RULE_UDP0 = 'rule.udp0~rule.udp0'
UNIT__RULE_TCPx__RULE_TCPx = 'rule.tcp{}~rule.tcp{}'


class LrCreator(BaseCreator):
    def __init__(self, lpr_uimod, cam_uimod, fusion_uimod, tcp_recipient_uimods, ledar_uimods,
                 location_uimods, ipcom_waga_uimods):
        """
        :type lpr_uimod: ph.UiMod
        :type cam_uimod: ph.UiMod
        :type fusion_uimod: ph.UiMod
        :type tcp_recipient_uimods: list of ph.UiMod
        :type ledar_uimods: list of ph.UiMod
        :type location_uimods: list of ph.UiMod
        :type ipcom_waga_uimods: list of ph.UiMod
        """
        BaseCreator.__init__(self)
        self._lpr_uimod = lpr_uimod
        self._cam_uimod = cam_uimod
        self._fusion_uimod = fusion_uimod
        self._tcp_recipient_uimods = tcp_recipient_uimods
        self._ledar_uimods = ledar_uimods
        self._location_uimods = location_uimods
        self._ipcom_waga_uimods = ipcom_waga_uimods

        self._lpr_dgmod = dg.DgMod.create(dg.EDGE_TYPE_LPR, lpr_uimod.pid, lpr_uimod.id)
        self._builder = UnityConfBuilder()

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return self._lpr_dgmod

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """
        spool_path = ph.get_spool_dir(self._lpr_uimod.pid)

        lpr_uiconf = uimod_to_newconf_map.get(self._lpr_uimod) or self._lpr_uimod.load_conf()
        lpr_pri = lpr_uiconf['ro']
        lpr_pub = lpr_uiconf['rw']
        cam_uiconf = uimod_to_newconf_map.get(self._cam_uimod) or self._cam_uimod.load_conf()
        cam_pri = cam_uiconf['ro']

        if self._cam_uimod.type == ph.UI_TYPE_MV_CAMERA:
            if cam_pri['raw-format'] == 'MONO8T':
                add_scene_react_dir = fs.join_path(
                    spool_path, 'mg{}-lr{}-scene'.format(self._cam_uimod.id, self._lpr_uimod.id))
            else:
                add_scene_react_dir = fs.join_path(
                    spool_path, 'rc{}-lr{}-scene'.format(self._cam_uimod.id, self._lpr_uimod.id))

        elif self._cam_uimod.type == ph.UI_TYPE_RTSP_CAMERA:
            add_scene_react_dir = fs.join_path(
                spool_path, 'rg{}-lr{}-scene'.format(self._cam_uimod.id, self._lpr_uimod.id))

        else:
            raise NotImplementedError

        image_wh = ph.CamDgStats.create(self._lpr_uimod.pid).get_stream_wh() or (640, 480)
        scene_event_jnode = [
            fs.join_path(spool_path, 'lr{}-tr{}-scene'.format(self._lpr_uimod.id, self._fusion_uimod.id))
        ]

        # enable gc
        if self._cam_uimod.type == ph.UI_TYPE_MV_CAMERA:
            if cam_uiconf['rw']['videostream']['control']['profile'] == 'auto-70-150':
                scene_event_jnode.append(fs.join_path(spool_path, 'lr{}-gc{}-scene'.format(
                    self._lpr_uimod.id, self._cam_uimod.id)))

        self._builder.unit(*essentials.create_unit__essentials__api(
            UNIT__ESSENTIALS0__API0, self._lpr_uimod.pid))
        self._builder.unit(*essentials.create_unit__essentials__counters(
            UNIT__ESSENTIALS0__COUNTERS0, spool_path))
        self._builder.unit(*bus.create_unit__media_server__media(
            UNIT__MEDIA_SERVER0__MEDIA0, add_scene_react_dir))
        self._builder.unit(*bus.create_unit__event_emitter__tx_track_bus(
            UNIT__EVENT_EMITTER0__TX_TRACK_BUS0, lpr_pri.get('translate-plate') or True,
            fs.join_path(spool_path, 'lr{0}-lr{0}-store'.format(self._lpr_uimod.id)),
            fs.join_path(spool_path, 'lr{}-tr{}-track'.format(self._lpr_uimod.id, self._fusion_uimod.id))))
        self._builder.unit(*bus.create_unit__event_emitter__tx_scene_bus(
            UNIT__EVENT_EMITTER0__TX_SCENE_BUS0, scene_event_jnode))
        self._builder.unit(*anpr.create_unit__anpr__anpr(
            UNIT__ANPR0__ANPR0, lpr_pri, lpr_pub, image_wh))
        
        self._builder.link_dist({UNIT__MEDIA_SERVER0__MEDIA0, UNIT__ESSENTIALS0__API0}, {UNIT__ANPR0__ANPR0})
        self._builder.link_dist({UNIT__ANPR0__ANPR0}, {UNIT__ESSENTIALS0__API0, UNIT__EVENT_EMITTER0__TX_SCENE_BUS0})
        self._builder.link(UNIT__ESSENTIALS0__COUNTERS0, UNIT__EVENT_EMITTER0__TX_SCENE_BUS0)

        units_seq = [UNIT__ANPR0__ANPR0]
        mmr_conf = lpr_pri.get(UNIT__MMR0__MMR0, None)
        if mmr_conf is not None:
            self._builder.unit(*mmr.create_unit__mmr___mmr(UNIT__MMR0__MMR0, mmr_conf))
            units_seq.append(UNIT__MMR0__MMR0)

        sbr_conf = lpr_pri.get(UNIT__SBR0__SBR0, None)
        if sbr_conf is not None:
            self._builder.unit(*sbr.create_unit__sbr___sbr(UNIT__SBR0__SBR0, sbr_conf))
            units_seq.append(UNIT__SBR0__SBR0)

        add_speedcam = lpr_pri.get('measure-lpr-speed') or False
        if add_speedcam:
            # TODO: technically can be empty (if videospeed enabled witout location-uimod)
            location_uimod = self._location_uimods[0]
            self._builder.unit(*speedcam.create_unit__speedcam___speedcam(
                UNIT__SPEEDCAM0__SPEEDCAM0, lpr_pri, self._cam_uimod.conf_path, location_uimod.conf_path))
            units_seq.append(UNIT__SPEEDCAM0__SPEEDCAM0)

        if len(self._ipcom_waga_uimods) == 1:
            waga_uimod = self._ipcom_waga_uimods[0]
            waga_conf = uimod_to_newconf_map.get(waga_uimod) or waga_uimod.load_conf()
            self._builder.unit(*waga.create_unit__ipcom___waga(UNIT__IPCOM0_WAGA0, waga_conf))
            units_seq.append(UNIT__IPCOM0_WAGA0)

        units_seq.append(UNIT__EVENT_EMITTER0__TX_TRACK_BUS0)
        self._builder.link_seq(units_seq)
        self._builder.link_dist([UNIT__ESSENTIALS0__COUNTERS0], units_seq)

        # create wiegand unit
        wiegand_conf = lpr_pri.get('wiegand-conf', None)
        if wiegand_conf:
            self._builder.unit(*wiegand.create_unit__wiegand___pcb(UNIT__WIEGAND_PCB0__RECIPIENT0, wiegand_conf))
            self._builder.unit(*recipient.create_unit__rule___rule2(UNIT__RULE_PCB0__RULE_PCB0, wiegand_conf['trigger']))

            self._builder.link_seq([UNIT__ANPR0__ANPR0, UNIT__RULE_PCB0__RULE_PCB0, UNIT__WIEGAND_PCB0__RECIPIENT0])
            self._builder.link(UNIT__ESSENTIALS0__COUNTERS0, UNIT__WIEGAND_PCB0__RECIPIENT0)

        # create wiegand-udp unit
        wiegand_udp_conf = lpr_pri.get('wiegand-udp-conf', None)
        if wiegand_udp_conf:
            self._builder.unit(*wiegand.create_unit__wiegand___udp(UNIT__WIEGAND_UDP0__RECIPIENT0, wiegand_udp_conf))

            self._builder.unit(*recipient.create_unit__rule___rule2(
                UNIT__RULE_UDP0__RULE_UDP0, wiegand_udp_conf['trigger']))

            self._builder.link_seq([UNIT__ANPR0__ANPR0, UNIT__RULE_UDP0__RULE_UDP0, UNIT__WIEGAND_UDP0__RECIPIENT0])
            self._builder.link(UNIT__ESSENTIALS0__COUNTERS0, UNIT__WIEGAND_UDP0__RECIPIENT0)

        tcp_index = 0
        for tcp_recipient_uimod in self._tcp_recipient_uimods:
            tcp_index += 1
            tcp_recipient_conf = uimod_to_newconf_map.get(tcp_recipient_uimod) or tcp_recipient_uimod.load_conf()

            self._builder.unit(*recipient.create_unit__recipient___vast2(tcp_recipient_uimod.ref, tcp_recipient_conf))

            rule_ref = UNIT__RULE_TCPx__RULE_TCPx.format(tcp_index, tcp_index)
            self._builder.unit(*recipient.create_unit__rule___rule(rule_ref, tcp_recipient_conf))

            self._builder.link_seq([UNIT__ANPR0__ANPR0, rule_ref, tcp_recipient_uimod.ref])
            self._builder.link(tcp_recipient_uimod.ref, UNIT__ESSENTIALS0__COUNTERS0)

        if len(self._ledar_uimods) == 1:
            leddar_uimod = self._ledar_uimods[0]
            leddar_conf = uimod_to_newconf_map.get(leddar_uimod) or leddar_uimod.load_conf()
            leddar_endpoint = fs.join_path(spool_path, 'lg{0}-sc{0}-track'.format(self._lpr_uimod.id))

            self._builder.unit(*leddar.create_unit__leddar___leddar(
                UNIT__LEDDAR0__LEDDAR0, leddar_conf, leddar_endpoint))

            self._builder.link(UNIT__MEDIA_SERVER0__MEDIA0, UNIT__LEDDAR0__LEDDAR0)
            self._builder.link(UNIT__LEDDAR0__LEDDAR0, UNIT__ESSENTIALS0__COUNTERS0)

    def complete(self):
        lr_dgmod = self._lpr_dgmod
        self._builder.build(lr_dgmod.conf_dir)
        lr_dgmod.save_setupstore_conf(active=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class LrDeleter:
    def __init__(self, dgmod):
        """
        :type dgmod: dg.DgMod
        """
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: LrDeleter | None
        """
        if entry_uimod.type == ph.UI_TYPE_LPR:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_LPR, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self):
        self._dgmod.uninstall()
