from datetime import datetime, tzinfo
from edge import eventlog
from edge.com import fs
from edge.webserver.sysconf import ph


# noinspection PyClassHasNoInit
class BsDgstats:
    _64K_PATH = '/var/spool/edge/system/zabbix/bs65535'

    @staticmethod
    def get_chan_stats(stats_path, chanref, tz=None):
        """
        :type stats_path: str
        :type chanref: str
        :type tz: tzinfo | None
        :rtype: dict | None
        """
        dgstats = fs.load_json3(stats_path, default=None)
        if dgstats:
            units_stats = dgstats['units']
            storages = filter(lambda unit_stats: '~chan.{}'.format(chanref) in units_stats['name'], units_stats)
            stats = filter(lambda unit_stats: '~stats.0' in units_stats['name'], units_stats)

            if not stats:
                eventlog.log_w('fail: NotFoundError')
                return

            if not storages:
                eventlog.log_w('fail: NotFoundError')
                return

            storage_jnode = storages[0]
            storage_stat_jnode = storage_jnode['stat']
            blobs_jnode = storage_stat_jnode['summary']['blobs']
            min_time = blobs_jnode['min-time']  # type: tuple
            max_time = blobs_jnode['max-time']  # type: tuple

            archive_jnode = storage_stat_jnode['summary']['archive']

            stat_jnode = stats[0]
            stat_stat_jnode = stat_jnode['stat']
            stat_mtime = stat_stat_jnode['mtime']  # type: tuple

            return {
                'time': datetime.fromtimestamp(stat_mtime[1], tz).isoformat(),
                'min-time': datetime.fromtimestamp(min_time[1], tz).isoformat(),
                'max-time': datetime.fromtimestamp(max_time[1], tz).isoformat(),
                'df': archive_jnode['df'],
                'du': archive_jnode['du'],
                'diru': archive_jnode['diru']
            }

        eventlog.log_w('fail: NotFoundError')

    @staticmethod
    def _get_connections_count(stats_path):
        """
        :type stats_path: str
        :rtype: int  | None
        """
        dgstats = fs.load_json3(stats_path, default=None)
        if dgstats:
            units_stats = dgstats['units']
            uds_servers = filter(lambda unit_stats: unit_stats['type'] == 'blobstore~uds-server', units_stats)

            for uds_server in uds_servers:
                count = uds_server.get('stat', dict()).get('client-count')
                if count is not None:
                    return count
                else:
                    return None

        eventlog.log_w('fail: NotFoundError')

    @classmethod
    def get_chan_stats_64k(cls, chanref, tz=None):
        """
        :type chanref: str
        :type tz: tzinfo | None
        :rtype: dict | None
        """
        return cls.get_chan_stats(cls._64K_PATH, chanref, tz)

    @classmethod
    def get_stats_path(cls, pid=None):
        """
        :type pid: str | None
        :rtype: str
        """
        if pid is not None:
            spool_path = ph.get_spool_dir(pid)
            stat_path = fs.join_path(spool_path, 'zabbix', 'bs0')
        else:
            stat_path = cls._64K_PATH
        return stat_path

    @classmethod
    def get_connections(cls, pid=None):
        """
        :type pid: str
        :rtype: None | int
        """
        return cls._get_connections_count(cls.get_stats_path(pid))

    @classmethod
    def get_db_cleanup_stats(cls):
        """
        :rtype: list ()
        """
        channels_stats = []
        stats_path = cls.get_stats_path()
        dgstats = fs.load_json3(stats_path, default=None)
        if dgstats:
            units_stats = dgstats['units']
            channels_jnode = filter(lambda unit_stats: unit_stats['type'] == 'blobstore~chan', units_stats)

            for channel_jnode in channels_jnode:
                blobs_jnode = channel_jnode['stat']['summary']['blobs']
                ch_ref = channel_jnode['name'].split('.')[-1]
                jnode = {'channel': ch_ref, 'min-time': blobs_jnode['min-time'][1], 'max-time': blobs_jnode['max-time'][1]}
                channels_stats.append(jnode)

        return channels_stats

    @classmethod
    def get_cache_metric_64k(cls, chan_ref, metric_name):
        """
        :type chan_ref: str
        :type metric_name: str

        :rtype: int | None
        """
        dgstats = fs.load_json3(cls._64K_PATH, default=None)
        if dgstats:
            units_stats = dgstats['units']
            storages = filter(lambda unit_stats: '~chan.{}'.format(chan_ref) in unit_stats['name'], units_stats)
            if not storages:
                eventlog.log_w('fail: NotFoundError')
                return None

            storage_jnode = storages[0]
            cache_jnode = storage_jnode['stat']['summary']['cache']
            return cache_jnode.get(metric_name)
